"""Authentication module."""

from sdv_installer.authentication.authentication import (
    authenticate,
    authenticate_user_and_license_key,
)

__all__ = ('authenticate', 'authenticate_user_and_license_key')
